package main

import (
	"time"
)

//Commands
const (
	Clone        string = "clone"
	Commit       string = "commit"
	Pull         string = "pull"
	Push         string = "push"
	CreateBranch string = "create_branch"
	DeleteBranch string = "delete_branch"
	Checkout     string = "checkout"
	Status       string = "status"
	AddRemote    string = "add_remote"
	DeleteRemote string = "delete_remote"
	FileCheckout string = "file_checkout"
)

//Command state
const (
	SuccessState string = "Success"
	FailedState  string = "Failed"
	RunningState string = "Running"
)

const TestDir string = "./test_dir"

const (
	DefaultRemoteName string = "origin"
)

type BaseCommandConfig struct {
	CommandType string `json:"commandType"`
	Path        string `json:"path"`
}

type GitCommandExecutionStateUpdateDTO struct {
	GitOpExecutionState string `json:"executionState"`
	Message             string `json:"message"`
	CommitHash          string `json:"commitHash"`
	Branch              string `json:"branch"`
}

type ExecutionState struct {
	GitOpExecutionState string `json:"executionState"`
}

type CloneCommandConfiguration struct {
	CommandType string `json:"commandType"`
	Url         string `json:"url"`
	Provider    string `json:"provider"`
	Branch      string `json:"branchName"`
}

type CommitterSignature struct {
	Name  string    `json:"name"`
	Email string    `json:"email"`
	Time  time.Time `json:"time"`
}

type CommitDTO struct {
	Name       string    `json:"name"`
	Email      string    `json:"email"`
	Time       time.Time `json:"date"`
	CommitHash string    `json:"hash"`
	Message    string    `json:"message"`
}

type CommitCommandConfiguration struct {
	Message            string             `json:"message"`
	All                bool               `json:"all"`
	CommitterSignature CommitterSignature `json:"committer"`
	Files              []string           `json:"files"`
}

type PushCommandConfiguration struct {
	RemoteName string `json:"remoteName"`
	Force      bool   `json:"force"`
}

type PullCommandConfiguration struct {
	RepoPath           string             `json:"path"`
	BranchName         string             `json:"branchName"`
	RemoteName         string             `json:"remoteName"`
	Force              bool               `json:"force"`
	CommitterSignature CommitterSignature `json:"committer"`
}

type CreateBranchCommandConfiguration struct {
	BranchName string `json:"branchName"`
	Checkout   bool   `json:"checkout"`
}

type DeleteBranchCommandConfiguration struct {
	BranchName string `json:"branchName"`
}

type CheckoutCommandConfiguration struct {
	BranchName string `json:"branchName"`
	Force      bool   `json:"force"`
	Commit     string `json:"commit"`
}

type AddRemoteCommandConfiguration struct {
	RemoteName string `json:"remoteName"`
	RemoteURL  string `json:"remoteUrl"`
}

type DeleteRemoteCommandConfiguration struct {
	RemoteName string `json:"remoteName"`
}

type RemoteDTO struct {
	RemoteName string `json:"name"`
	Url        string `json:"url"`
}

type BranchCommitsDTO struct {
	Commits []CommitDTO `json:"commits"`
}

type FileStatus struct {
	File   string `json:"file"`
	Status string `json:"status"`
	Extra  string `json:"extra"`
}

type StatusDTO struct {
	Files []FileStatus `json:"status"`
}

type FileCheckoutCommandConfiguration struct {
	Files []string `json:"files"`
}
