#!/bin/bash

CERTS_DIR=$MATERIAL_DIRECTORY
TSTORE_FILE=$CERTS_DIR/${HADOOP_USERNAME}__tstore.jks
KSTORE_FILE=$CERTS_DIR/${HADOOP_USERNAME}__kstore.jks
KEY_FILE=$CERTS_DIR/${HADOOP_USERNAME}__cert.key

KEY=$( cat ${KEY_FILE} )

#1. generate pem certificates bundle from the keystore.jks file
keytool -importkeystore -srckeystore $KSTORE_FILE -destkeystore $PEMS_DIR/${HADOOP_USERNAME}__keystore.p12 -deststoretype PKCS12 -srcstorepass $KEY -keypass $KEY -deststorepass $KEY
openssl pkcs12 -legacy -nokeys -in $PEMS_DIR/${HADOOP_USERNAME}__keystore.p12 -out $PEMS_DIR/${HADOOP_USERNAME}_certificate_bundle.pem -passin pass:$KEY

#2. generate root ca pem from the tstore.jks file file
keytool -importkeystore -srckeystore $TSTORE_FILE -destkeystore $PEMS_DIR/${HADOOP_USERNAME}__tstore.p12 -deststoretype PKCS12 -srcstorepass $KEY -keypass $KEY -deststorepass $KEY
openssl pkcs12 -legacy -nokeys -in $PEMS_DIR/${HADOOP_USERNAME}__tstore.p12 -out $PEMS_DIR/${HADOOP_USERNAME}_root_ca.pem -passin pass:$KEY

#3 extract private key from the keystore
openssl pkcs12 -legacy -info -in $PEMS_DIR/${HADOOP_USERNAME}__keystore.p12 -nodes -nocerts > $PEMS_DIR/${HADOOP_USERNAME}_private_key.pem -passin pass:$KEY

#4. verify that files have been generated
CERTIFICATES_BUNDLE=$PEMS_DIR/${HADOOP_USERNAME}_certificate_bundle.pem
if [ ! -f ${CERTIFICATES_BUNDLE} ]; then
echo "Failed to generate certificates bundle for project"
exit 4
fi

ROOT_CA=$PEMS_DIR/${HADOOP_USERNAME}_root_ca.pem
if [ ! -f ${ROOT_CA} ]; then
echo "Failed to generate root ca for project"
exit 5
fi

PRIVATE_KEY=$PEMS_DIR/${HADOOP_USERNAME}_private_key.pem
if [ ! -f ${PRIVATE_KEY} ]; then
echo "Failed to generate private key for project"
exit 6
fi

chmod 644 $ROOT_CA
chmod 644 $CERTIFICATES_BUNDLE
chmod 644 $PRIVATE_KEY

rm $PEMS_DIR/${HADOOP_USERNAME}__keystore.p12
rm $PEMS_DIR/${HADOOP_USERNAME}__tstore.p12