package main

import (
	"encoding/json"
	"errors"
	"fmt"
	"github.com/go-git/go-git/v5"
)

type FileActions struct {
	repository *git.Repository
}

func (f *FileActions) ExecuteFileAction() (status git.Status, err error) {
	switch commandType {
	case FileCheckout:
		var config FileCheckoutCommandConfiguration
		json.Unmarshal([]byte(commandConfig), &config)
		status, err = f.Checkout(config)
	default:
		err = errors.New("Unknown file action: " + commandType)
	}

	if updateHopsworks {
		message := ""
		finalState := SuccessState
		if err != nil {
			finalState = FailedState
			message = fmt.Sprintf("%s failed. %s", commandType, err.Error())
			logger.LogError(message, nil)
		} else {
			repoActions := &RepositoryActions{r: f.repository}
			statusDTO, _ := repoActions.Status()
			jsonStr, _ := json.Marshal(statusDTO)
			message = string(jsonStr)
			logger.LogInfo(fmt.Sprintf("%s was successfull. New status is %s", commandType, message), nil)
		}
		UpdateFinalRepositoryState(f.repository, message, finalState)
	}

	return status, err
}

func (f *FileActions) Checkout(fileCheckoutCommandConfig FileCheckoutCommandConfiguration) (status git.Status, err error) {
	files := fileCheckoutCommandConfig.Files
	w, err := f.repository.Worktree()
	if err != nil {
		return status, err
	}
	status, err = w.Status()
	for _, path := range files {
		fs := status.File(path)
		if (fs.Staging == git.Unmodified && fs.Worktree == git.Unmodified) || (fs.Staging == git.Untracked && fs.Worktree == git.Untracked) {
			continue
		} else {
			ExecuteOnPath(repositoryPath, "git checkout "+path)
		}
	}
	status, err = w.Status()
	return status, err
}
