package main

import (
	"github.com/stretchr/testify/require"
	"testing"
)

func TestFileActions_Checkout(t *testing.T) {
	r, _, _ := CloneRepoTest(t, "file_checkout")
	err := ExecuteOnPath(repositoryPath, "rm README.md")
	require.Nil(t, err)
	repoActions := &RepositoryActions{r: r}
	status, err := repoActions.Status()
	require.Nil(t, err)
	require.EqualValues(t, len(status.Files), 1)
	f := status.Files[0]
	require.Equal(t, f.File, "README.md")
	fileActions := &FileActions{repository: r}
	newStatus, err := fileActions.Checkout(FileCheckoutCommandConfiguration{Files: []string{"README.md"}})
	require.Nil(t, err)
	require.Equal(t, newStatus.IsClean(), true)
}

func TestFileActions_Checkout_Unmodified(t *testing.T) {
	r, _, _ := CloneRepoTest(t, "file_checkout_unmodified")
	repoActions := &RepositoryActions{r: r}
	status, err := repoActions.Status()
	require.Nil(t, err)
	require.EqualValues(t, len(status.Files), 0)
	fileActions := &FileActions{repository: r}
	newStatus, err := fileActions.Checkout(FileCheckoutCommandConfiguration{Files: []string{"README.md"}})
	require.Nil(t, err)
	require.Equal(t, newStatus.IsClean(), true)
}

func TestFileActions_Checkout_Untracked(t *testing.T) {
	testFile := "blah.txt"
	r, _, _ := CloneRepoTest(t, "file_checkout_untracked")
	repoActions := &RepositoryActions{r: r}
	status, err := repoActions.Status()
	require.Nil(t, err)
	require.EqualValues(t, len(status.Files), 0)
	//create new file
	err = ExecuteOnPath(repositoryPath, "touch "+testFile)
	require.Nil(t, err)
	oldStatusDto, err := repoActions.Status()
	require.EqualValues(t, len(oldStatusDto.Files), 1)
	f1 := oldStatusDto.Files[0]
	require.Equal(t, f1.File, testFile)
	fileActions := &FileActions{repository: r}
	newStatus, err := fileActions.Checkout(FileCheckoutCommandConfiguration{Files: []string{testFile}})
	require.Nil(t, err)
	require.Equal(t, newStatus.IsClean(), false)
	newStatusDto, err := repoActions.Status()
	require.Nil(t, err)
	require.EqualValues(t, len(newStatusDto.Files), 1)
	f2 := newStatusDto.Files[0]
	require.EqualValues(t, f1, f2)
}
