package main

import (
	"fmt"
	"os"
	"runtime"

	nested "github.com/antonfisher/nested-logrus-formatter"
	"github.com/sirupsen/logrus"
)

// bunch of constants for logging
const (
	Message      = "msg"
	Line         = "line"
	ExecutionId  = "executionid"
	RepositoryId = "repositoryid"
	ProjectId    = "projectid"
)

type Logger struct{}

var ReportCaller = true

func InitLogger(level string, reportCaller bool) *Logger {
	ReportCaller = reportCaller
	lvl, err := logrus.ParseLevel(level)
	if err != nil {
		logrus.Errorf("Invalid logs level %s ", level)
		lvl = logrus.ErrorLevel
	}

	// Only logs the warning severity or above.
	logrus.SetLevel(lvl)

	logrus.SetFormatter(&nested.Formatter{
		HideKeys:       false,
		NoFieldsColors: true,
		FieldsOrder:    []string{ProjectId, RepositoryId, ExecutionId},
	})

	logrus.SetOutput(os.Stdout)

	return &Logger{}
}

type Fields logrus.Fields

func (l *Logger) LogInfo(msg string, f Fields) {
	l.LogMessage(logrus.InfoLevel, msg, f)
}

func (l *Logger) LogWarn(msg string, f Fields) {
	l.LogMessage(logrus.WarnLevel, msg, f)
}

func (l *Logger) LogError(msg string, f Fields) {
	l.LogMessage(logrus.ErrorLevel, msg, f)
}

func (l *Logger) LogFatal(msg string, f Fields) {
	l.LogMessage(logrus.FatalLevel, msg, f)
}

func (l *Logger) LogMessage(lvl logrus.Level, msg string, f Fields) {
	if f == nil {
		f = Fields{}
	}
	if ReportCaller {
		_, file, line, _ := runtime.Caller(2)
		f[Line] = fmt.Sprintf("%s:%d", file, line)
	}

	if updateHopsworks {
		f[ExecutionId] = executionId
		f[RepositoryId] = repositoryId
		f[ProjectId] = projectId
	}

	switch lvl {
	case logrus.FatalLevel:
		logrus.WithFields(logrus.Fields(f)).Fatal(msg)
	case logrus.ErrorLevel:
		logrus.WithFields(logrus.Fields(f)).Error(msg)
	case logrus.WarnLevel:
		logrus.WithFields(logrus.Fields(f)).Warn(msg)
	case logrus.InfoLevel:
		logrus.WithFields(logrus.Fields(f)).Info(msg)
	default:
		logrus.WithFields(logrus.Fields(f)).Info(msg)
	}
}
