package main

import (
	"encoding/json"
	"errors"
	"fmt"
	"github.com/go-git/go-git/v5"
	go_git_config "github.com/go-git/go-git/v5/config"
)

type RemoteActions struct {
	repository *git.Repository
}

func (remote *RemoteActions) ExecuteRemoteAction(command string) (remotes []RemoteDTO, err error) {
	switch command {
	case AddRemote:
		var config AddRemoteCommandConfiguration
		json.Unmarshal([]byte(commandConfig), &config)
		remotes, err = remote.Add(config.RemoteName, config.RemoteURL)
	case DeleteRemote:
		var config DeleteRemoteCommandConfiguration
		json.Unmarshal([]byte(commandConfig), &config)
		remotes, err = remote.Delete(config.RemoteName)
	default:
		remotes, _ = remote.GetRemotes()
		err = errors.New(fmt.Sprintf("Unknown remote action %s", command))
	}
	if updateHopsworks {
		jsonStr, _ := json.Marshal(&remotes)
		finalState := SuccessState
		message := string(jsonStr)
		if err != nil {
			finalState = FailedState
			message = err.Error()
			logger.LogError(message, nil)
		} else {
			logger.LogInfo(fmt.Sprintf("%s was successfull. New remotes are %s", command, message), nil)
		}
		UpdateFinalRepositoryState(remote.repository, message, finalState)
	}
	return remotes, err
}

func (remote *RemoteActions) Add(remoteName string, remoteUrl string) (remotes []RemoteDTO, err error) {
	logger.LogInfo(fmt.Sprintf("Adding remote. Remote name: %s, remote url: %s", remoteName, remoteUrl), nil)
	// Add a new remote, with the default fetch refspec
	_, err = remote.repository.CreateRemote(&go_git_config.RemoteConfig{
		Name: remoteName,
		URLs: []string{remoteUrl},
	})
	if err != nil {
		logger.LogError(err.Error(), nil)
	} else {
		logger.LogInfo(fmt.Sprintf("Successfully added remote %s. Url %s", remoteName, remoteUrl), nil)
	}
	//Get new remote list
	remotes, _ = remote.GetRemotes()
	return remotes, err
}

func (remote *RemoteActions) Delete(remoteName string) (remotes []RemoteDTO, err error) {
	logger.LogInfo(fmt.Sprintf("deleting remote %s", remoteName), nil)
	err = remote.repository.DeleteRemote(remoteName)
	if err != nil {
		logger.LogError(err.Error(), nil)
	} else {
		logger.LogInfo(fmt.Sprintf("Successfully deleted remote %s", remoteName), nil)
	}
	remotes, _ = remote.GetRemotes()
	return remotes, err
}

func (remote *RemoteActions) GetRemotes() (dtos []RemoteDTO, err error) {
	// List remotes from a repository
	logger.LogInfo("git remotes -v", nil)
	remotes, err := remote.repository.Remotes()
	if err != nil {
		logger.LogError(err.Error(), nil)
		return dtos, err
	}
	for _, remote := range remotes {
		dtos = append(dtos, RemoteDTO{
			RemoteName: remote.Config().Name,
			Url:        (remote.Config().URLs)[0],
		})
	}
	return dtos, nil
}
